import type { LoggerService as BaseLoggerService } from '@nestjs/common'
import { log } from '@pulumi/pulumi'
import type { Color } from 'colorette'
import { createColors } from 'colorette'
import { format } from 'util'

import { LogLevels } from './logger.constants'

const colors = createColors({ useColor: true })

export class LoggerService implements BaseLoggerService {
  private readonly color: Record<LogLevels, Color> = {
    [LogLevels.ERROR]: colors.red,
    [LogLevels.WARN]: colors.yellow,
    [LogLevels.INFO]: colors.green,
    [LogLevels.DEBUG]: colors.magenta
  }

  constructor (private readonly context?: string) {}

  public async error (message: any, trace?: string, context?: string): Promise<void> {
    await log.error(this.printf(LogLevels.ERROR, message, context))

    if (trace) {
      await log.debug(this.printf(LogLevels.DEBUG, trace, context))
    }
  }

  public async warn (message: any, context?: string): Promise<void> {
    return log.warn(this.printf(LogLevels.WARN, message, context))
  }

  public async log (message: any, context?: string): Promise<void> {
    return log.info(this.printf(LogLevels.INFO, message, context))
  }

  public async debug (message: any, context?: string): Promise<void> {
    return log.debug(this.printf(LogLevels.DEBUG, message, context))
  }

  private printf (level: LogLevels, message: string | [string, ...any[]], context?: string): string {
    const prefix = `${this.color[level](`[${this.context ?? context}]`)} `

    if (typeof message === 'string') {
      return prefix + colors.yellow(message)
    }

    message[0] = prefix + message[0]

    return colors.yellow(format(...message))
  }
}
